import string
import random


class GreinLoaderUtils:
    def __init__(self, gse_id: str = None):
        """Initialize a new utils function. If gse_id is set, this injected in the
           respective data values.

        :param gse_id: The GSE id to load, defaults to None
        :type gse_id: str, optional
        """
        self.gse_id = gse_id

    def ui_init_parameter(self):
        return '["1#0|m|{\\"method\\":\\"init\\",\\"data\\":{\\"modalheatmap_regsig\\":false,\\"grin\\":\\"datasets\\",\\"tab2\\":\\"summary\\",\\"preplots\\":\\"corr\\",\\"heatvis\\":\\"static\\",\\"fir_or_scr\\":\\"fitinscreen\\",\\"pcap\\":\\"pca2\\",\\"heatmodal_regsig\\":\\"static_mod_regsig\\",\\"fit_or_scr_mod_regsig\\":\\"fitinscreen_mod_regsig\\",\\"signature\\":\\"create_sig\\",\\"cr_reg_sig\\":\\"meta_reg\\",\\"sigtab\\":\\"meta_powers\\",\\"process\\":\\"log\\",\\"user_geo_bs:shiny.action\\":0,\\"start_process:shiny.action\\":0,\\"sample_ont_ser_bs:shiny.action\\":0,\\"process_log_bs:shiny.action\\":0,\\"process_log:shiny.action\\":0,\\"load_counts:shiny.action\\":0,\\"draw_heat:shiny.action\\":0,\\"draw_heatscr:shiny.action\\":0,\\"draw_heatint:shiny.action\\":0,\\"draw_pca:shiny.action\\":0,\\"draw_pca3:shiny.action\\":0,\\"draw_tsne2:shiny.action\\":0,\\"draw_tsne3:shiny.action\\":0,\\"heat_stat_bs:shiny.action\\":0,\\"heat_int_bs:shiny.action\\":0,\\"heatmap_mod_regsig:shiny.action\\":0,\\"draw_heatfit_regsig:shiny.action\\":0,\\"draw_heatscr_regsig:shiny.action\\":0,\\"draw_heatint_regsig:shiny.action\\":0,\\"draw_ma_regsig:shiny.action\\":0,\\"gen_pow_mod_btn:shiny.action\\":0,\\"search_mod_bs:shiny.action\\":0,\\"de_sig_bs:shiny.action\\":0,\\"de_sig_bs2:shiny.action\\":0,\\"power_bs:shiny.action\\":0,\\"detect_bs:shiny.action\\":0,\\"search_gene_bs:shiny.action\\":0,\\"show_log:shiny.action\\":0,\\"counts_choice\\":\\"Raw\\",\\"download_type_counts\\":\\"Gene level\\",\\"download_type_fit\\":\\"png\\",\\"download_type_scr\\":\\"png\\",\\"sub_prop_tsne2\\":\\"No\\",\\"sub_sam_pca\\":\\"No\\",\\"fact_choice\\":\\"All samples\\",\\"sub_sam\\":\\"No\\",\\"download_type_fit_mod_regsig\\":\\"png\\",\\"download_type_scr_mod_regsig\\":\\"png\\",\\"gen_det_mod_type\\":\\"All\\",\\"pow_fact_choice\\":\\"All samples\\",\\"pow_sub_sample2\\":\\"No\\",\\"sample_ont_ser\\":[],\\"geo_next_lp\\":\\"\\",\\"geo_current\\":\\"\\",\\"geo_next\\":\\"\\",\\"user_geo\\":\\"\\",\\"search_mod_gene\\":\\"\\",\\"search_gene\\":\\"\\",\\".clientdata_output_samplesize_den_width\\":440,\\".clientdata_output_samplesize_den_height\\":255,\\".clientdata_output_study_stat_width\\":575,\\".clientdata_output_study_stat_height\\":255,\\".clientdata_output_sample_stat_width\\":595,\\".clientdata_output_sample_stat_height\\":255,\\".clientdata_output_stepbystep_hidden\\":true,\\".clientdata_output_samplesize_den_hidden\\":false,\\".clientdata_output_study_stat_hidden\\":false,\\".clientdata_output_sample_stat_hidden\\":false,\\".clientdata_output_warn_hidden\\":false,\\".clientdata_output_warn2_hidden\\":false,\\".clientdata_output_warn3_hidden\\":false,\\".clientdata_output_warn4_hidden\\":false,\\".clientdata_output_warn5_hidden\\":false,\\".clientdata_output_warn_ont_hidden\\":false,\\".clientdata_output_warn_ont2_hidden\\":false,\\".clientdata_output_datatable_hidden\\":false,\\".clientdata_output_geo_acc_ui_hidden\\":true,\\".clientdata_output_metadata_column_hidden\\":true,\\".clientdata_output_full_metacol_hidden\\":true,\\".clientdata_output_downloadmeta_hidden\\":true,\\".clientdata_output_counts_sample_ui_hidden\\":true,\\".clientdata_output_downloadcounts_hidden\\":true,\\".clientdata_output_cor_sample_ui_hidden\\":true,\\".clientdata_output_show_grp_ui_hidden\\":true,\\".clientdata_output_corr_prop_hidden\\":true,\\".clientdata_output_draw_cor_ui_hidden\\":true,\\".clientdata_output_dens_sample_ui_hidden\\":true,\\".clientdata_output_draw_dens_ui_hidden\\":true,\\".clientdata_output_heat_prop_hidden\\":true,\\".clientdata_output_cluster_meth_hidden\\":true,\\".clientdata_output_cluster_genes_hidden\\":true,\\".clientdata_output_download_hidden\\":true,\\".clientdata_output_downloadscr_hidden\\":true,\\".clientdata_output_pca_prop_hidden\\":true,\\".clientdata_output_subprop2_ui_hidden\\":true,\\".clientdata_output_subprop_ui_hidden\\":true,\\".clientdata_output_legendplot_hidden\\":true,\\".clientdata_output_geo_summary_hidden\\":true,\\".clientdata_output_metadata_hidden\\":true,\\".clientdata_output_metadata_full_hidden\\":true,\\".clientdata_output_ontology_hidden\\":true,\\".clientdata_output_counts_warn_hidden\\":true,\\".clientdata_output_counts_text_hidden\\":true,\\".clientdata_output_counts_hidden\\":true,\\".clientdata_output_boxplot_counts_hidden\\":true,\\".clientdata_output_downloadqc_hidden\\":true,\\".clientdata_output_multiqc_hidden\\":true,\\".clientdata_output_corr_warn_hidden\\":true,\\".clientdata_output_corplot_ui_hidden\\":true,\\".clientdata_output_dens_warn_hidden\\":true,\\".clientdata_output_densityp_hidden\\":true,\\".clientdata_output_heat_warn_hidden\\":true,\\".clientdata_output_HeatStatic_hidden\\":true,\\".clientdata_output_HeatStaticscr_hidden\\":true,\\".clientdata_output_iheatmapui_hidden\\":true,\\".clientdata_output_downloadPCA2D_hidden\\":true,\\".clientdata_output_p2D_ui_hidden\\":true,\\".clientdata_output_pca_sam_warn_hidden\\":true,\\".clientdata_output_pca3Dplot_ui_hidden\\":true,\\".clientdata_output_t2Dplot_ui_hidden\\":true,\\".clientdata_output_tsne3Dplot_ui_hidden\\":true,\\".clientdata_output_geo_acc2_ui_hidden\\":true,\\".clientdata_output_metadata_analysis_hidden\\":true,\\".clientdata_output_group1_hidden\\":true,\\".clientdata_output_group2_hidden\\":true,\\".clientdata_output_ana_type_hidden\\":true,\\".clientdata_output_cov_hidden\\":true,\\".clientdata_output_meta_ui_hidden\\":true,\\".clientdata_output_gen_reg_sig_ui_hidden\\":true,\\".clientdata_output_heat_typeui_mod_regsig_hidden\\":true,\\".clientdata_output_heat_prop_mod_regsig_hidden\\":true,\\".clientdata_output_cluster_meth_mod_regsig_hidden\\":true,\\".clientdata_output_cluster_genes_mod_regsig_hidden\\":true,\\".clientdata_output_downloadfit_mod_regsig_hidden\\":true,\\".clientdata_output_downloadscr_mod_regsig_hidden\\":true,\\".clientdata_output_fdrui_mod_regsig_hidden\\":true,\\".clientdata_output_effect_ma_mod_ui_hidden\\":true,\\".clientdata_output_power_mod_ui_hidden\\":true,\\".clientdata_output_alpha_mod_ui_hidden\\":true,\\".clientdata_output_fitscreen_mod_regsig_hidden\\":true,\\".clientdata_output_scroll_mod_regsig_hidden\\":true,\\".clientdata_output_iheatmapui_mod_regsig_hidden\\":true,\\".clientdata_output_downloadMA_hidden\\":true,\\".clientdata_output_maplotui_mod_regsig_hidden\\":true,\\".clientdata_output_ma_mod_text_hidden\\":true,\\".clientdata_output_maplot_clickedpoints_hidden\\":true,\\".clientdata_output_gene_mod_warn_hidden\\":true,\\".clientdata_output_gene_sym_mod_warn_hidden\\":true,\\".clientdata_output_gene_sym_mod_green_hidden\\":true,\\".clientdata_output_det_mod_label_hidden\\":true,\\".clientdata_output_gen_det_mod_dl_hidden\\":true,\\".clientdata_output_plotdetui_mod_regsig_hidden\\":true,\\".clientdata_output_det_mod_text_hidden\\":true,\\".clientdata_output_det_mod_gene_clickedpoints_hidden\\":true,\\".clientdata_output_downloadsigdata_hidden\\":true,\\".clientdata_output_ui_reg_ilincs_option_hidden\\":true,\\".clientdata_output_power_sig_ui_hidden\\":true,\\".clientdata_output_alpha_sig_ui_hidden\\":true,\\".clientdata_output_dynamiclink_hidden\\":true,\\".clientdata_output_prop_power_ui_hidden\\":true,\\".clientdata_output_group1_pow_ui_hidden\\":true,\\".clientdata_output_group2_pow_ui_hidden\\":true,\\".clientdata_output_meta_pow_ui_hidden\\":true,\\".clientdata_output_gen_meta_pow_ui_hidden\\":true,\\".clientdata_output_samples1_hidden\\":true,\\".clientdata_output_samples2_hidden\\":true,\\".clientdata_output_effect_hidden\\":true,\\".clientdata_output_alpha_hidden\\":true,\\".clientdata_output_depth_ui_hidden\\":true,\\".clientdata_output_power_ui_hidden\\":true,\\".clientdata_output_det_effect_ui_hidden\\":true,\\".clientdata_output_fdr_ui_hidden\\":true,\\".clientdata_output_prop_detect_ui_hidden\\":true,\\".clientdata_output_group1_detect_ui_hidden\\":true,\\".clientdata_output_group2_detect_ui_hidden\\":true,\\".clientdata_output_gen_pow_btn_ui_hidden\\":true,\\".clientdata_output_gen_detect_btn_ui_hidden\\":true,\\".clientdata_output_sig1_warn_hidden\\":true,\\".clientdata_output_rank_warn_hidden\\":true,\\".clientdata_output_level_warn_hidden\\":true,\\".clientdata_output_sig_meta_hidden\\":true,\\".clientdata_output_sig_data_hidden\\":true,\\".clientdata_output_meta_pow_txt_hidden\\":true,\\".clientdata_output_meta_power_warn_hidden\\":true,\\".clientdata_output_metadata_pow_table_hidden\\":true,\\".clientdata_output_power1_warn_hidden\\":true,\\".clientdata_output_powerplot_hidden\\":true,\\".clientdata_output_gene_warn_hidden\\":true,\\".clientdata_output_gene_sym_warn_hidden\\":true,\\".clientdata_output_gene_sym_green_hidden\\":true,\\".clientdata_output_power2_warn_hidden\\":true,\\".clientdata_output_detect_gene_ui_hidden\\":true,\\".clientdata_output_gen_det_text_hidden\\":true,\\".clientdata_output_detect_gene_tab_hidden\\":true,\\".clientdata_output_steps_hidden\\":true,\\".clientdata_output_process_log_text_hidden\\":true,\\".clientdata_pixelratio\\":1,\\".clientdata_url_protocol\\":\\"http:\\",\\".clientdata_url_hostname\\":\\"www.ilincs.org\\",\\".clientdata_url_port\\":\\"\\",\\".clientdata_url_pathname\\":\\"/apps/grein/\\",\\".clientdata_url_search\\":\\"?gse=' + self.gse_id + '\\",\\".clientdata_url_hash_initial\\":\\"\\",\\".clientdata_url_hash\\":\\"\\",\\".clientdata_singletons\\":\\"991a9dee8b6f41039f0d48e0f3cb481dfef69248,abbc5191535f7e95ba1254171676097ac26b5edc,2581d36bc09732ee052ca2bcc5a7ae809019206f,2936dd0d01d731dd90358375bdaef9c44ce3b494,603e796bcfc2ab3685167d58c426f64c15a95192\\",\\".clientdata_allowDataUriScheme\\":true}}"]'

    def method_update_parameter(self):
        return '["2#0|m|{\\"method\\":\\"update\\",\\"data\\":{\\".clientdata_output_warn_hidden\\":true,\\".clientdata_output_warn3_hidden\\":true,\\".clientdata_output_warn4_hidden\\":true,\\".clientdata_output_warn5_hidden\\":true}}"]'

    def client_parameter(self):
        return '["3#0|m|{\\"method\\":\\"update\\",\\"data\\":{\\".clientValue-default-plotlyCrosstalkOpts\\":{\\"on\\":\\"plotly_click\\",\\"persistent\\":false,\\"dynamic\\":false,\\"selectize\\":false,\\"opacityDim\\":0.2,\\"selected\\":{\\"opacity\\":1},\\"debounce\\":0,\\"color\\":[]},\\"datatable_rows_current\\":[],\\"datatable_rows_all\\":[],\\"datatable_state\\":null,\\"datatable_search\\":\\"\\",\\"datatable_search_columns\\":[\\"\\",\\"\\",\\"\\",\\"\\",\\"\\"],\\"datatable_cell_clicked\\":{},\\"geo_next_lp\\":\\"GSE103221\\",\\"grin\\":\\"explore\\",\\"plotly_afterplot-A\\":\\"\\\\\\"sample_stat\\\\\\"\\",\\".clientdata_output_samplesize_den_hidden\\":true,\\".clientdata_output_study_stat_hidden\\":true,\\".clientdata_output_sample_stat_hidden\\":true,\\".clientdata_output_warn2_hidden\\":true,\\".clientdata_output_warn_ont_hidden\\":true,\\".clientdata_output_warn_ont2_hidden\\":true,\\".clientdata_output_datatable_hidden\\":true,\\".clientdata_output_geo_acc_ui_hidden\\":false,\\".clientdata_output_cor_sample_ui_hidden\\":false,\\".clientdata_output_show_grp_ui_hidden\\":false,\\".clientdata_output_corr_prop_hidden\\":false,\\".clientdata_output_draw_cor_ui_hidden\\":false,\\".clientdata_output_geo_summary_hidden\\":false}}"]'

    def stream_dataset_parameter(self):
        return '["7#0|m|{\\"method\\":\\"update\\",\\"data\\":{\\"geo_acc\\":\\"' + self.gse_id + '\\"}}"]'

    def description_formdata(self, no_samples = 100):
        return f"draw=1&columns[0][data]=0&columns[0][name]&columns[0][searchable]=true&columns[0][orderable]=false&columns[0][search][value]&columns[0][search][regex]=false&columns[1][data]=1&columns[1][name]&columns[1][searchable]=true&columns[1][orderable]=true&columns[1][search][value]&columns[1][search][regex]=false&start=0&length={no_samples}&search[value]&search[regex]=false&search[caseInsensitive]=true&search[smart]=true&escape=false"

    def metadata_labels_parameter(self):
        return '["B#0|m|{\\"method\\":\\"update\\",\\"data\\":{\\"tab2\\":\\"metadata\\",\\".clientdata_output_geo_summary_hidden\\":true,\\".clientdata_output_metadata_hidden\\":false,\\".clientdata_output_metadata_full_hidden\\":false,\\".clientdata_output_ontology_hidden\\":false}}"]'

    def count_matrix_parameter(self):
        return '["18#0|m|{\\"method\\":\\"update\\",\\"data\\":{\\"counts_rows_selected\\":[],\\"counts_rows_current\\":[],\\"counts_rows_all\\":[],\\"counts_state\\":null,\\"counts_search\\":\\"\\",\\"counts_cell_clicked\\":{},\\".clientdata_output_downloadcounts_hidden\\":false}}"]'

    def raw_form_start(self):
        return "draw=1&columns%5B0%5D%5Bdata%5D=0&columns%5B0%5D%5Bname%5D=&columns%5B0%5D%5Bsearchable%5D=true&columns%5B0%5D%5Borderable%5D=false&columns%5B0%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B0%5D%5Bsearch%5D%5Bregex%5D=false"

    def raw_form_end(self, no_samples = 100):
        return f"start=0&length={no_samples}&search%5Bvalue%5D=&search%5Bregex%5D=false&search%5BcaseInsensitive%5D=true&search%5Bsmart%5D=true&escape=true"

    def raw_form_column(self, n):
        return f"&columns%5B{n}%5D%5Bdata%5D={n}&columns%5B{n}%5D%5Bname%5D=&columns%5B{n}%5D%5Bsearchable%5D=true&columns%5B{n}%5D%5Borderable%5D=true&columns%5B{n}%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B{n}%5D%5Bsearch%5D%5Bregex%5D=false&"

    def overview_streaming(self):
        return '["1#0|m|{\\"method\\":\\"init\\",\\"data\\":{\\"modalheatmap_regsig\\":false,\\"grin\\":\\"datasets\\",\\"tab2\\":\\"summary\\",\\"preplots\\":\\"corr\\",\\"heatvis\\":\\"static\\",\\"fir_or_scr\\":\\"fitinscreen\\",\\"pcap\\":\\"pca2\\",\\"heatmodal_regsig\\":\\"static_mod_regsig\\",\\"fit_or_scr_mod_regsig\\":\\"fitinscreen_mod_regsig\\",\\"signature\\":\\"create_sig\\",\\"cr_reg_sig\\":\\"meta_reg\\",\\"sigtab\\":\\"meta_powers\\",\\"process\\":\\"log\\",\\"user_geo_bs:shiny.action\\":0,\\"start_process:shiny.action\\":0,\\"sample_ont_ser_bs:shiny.action\\":0,\\"process_log_bs:shiny.action\\":0,\\"process_log:shiny.action\\":0,\\"load_counts:shiny.action\\":0,\\"draw_heat:shiny.action\\":0,\\"draw_heatscr:shiny.action\\":0,\\"draw_heatint:shiny.action\\":0,\\"draw_pca:shiny.action\\":0,\\"draw_pca3:shiny.action\\":0,\\"draw_tsne2:shiny.action\\":0,\\"draw_tsne3:shiny.action\\":0,\\"heat_stat_bs:shiny.action\\":0,\\"heat_int_bs:shiny.action\\":0,\\"heatmap_mod_regsig:shiny.action\\":0,\\"draw_heatfit_regsig:shiny.action\\":0,\\"draw_heatscr_regsig:shiny.action\\":0,\\"draw_heatint_regsig:shiny.action\\":0,\\"draw_ma_regsig:shiny.action\\":0,\\"gen_pow_mod_btn:shiny.action\\":0,\\"search_mod_bs:shiny.action\\":0,\\"de_sig_bs:shiny.action\\":0,\\"de_sig_bs2:shiny.action\\":0,\\"power_bs:shiny.action\\":0,\\"detect_bs:shiny.action\\":0,\\"search_gene_bs:shiny.action\\":0,\\"show_log:shiny.action\\":0,\\"counts_choice\\":\\"Raw\\",\\"download_type_counts\\":\\"Gene level\\",\\"download_type_fit\\":\\"png\\",\\"download_type_scr\\":\\"png\\",\\"sub_prop_tsne2\\":\\"No\\",\\"sub_sam_pca\\":\\"No\\",\\"fact_choice\\":\\"All samples\\",\\"sub_sam\\":\\"No\\",\\"download_type_fit_mod_regsig\\":\\"png\\",\\"download_type_scr_mod_regsig\\":\\"png\\",\\"gen_det_mod_type\\":\\"All\\",\\"pow_fact_choice\\":\\"All samples\\",\\"pow_sub_sample2\\":\\"No\\",\\"sample_ont_ser\\":[],\\"geo_next_lp\\":\\"\\",\\"geo_current\\":\\"\\",\\"geo_next\\":\\"\\",\\"user_geo\\":\\"\\",\\"search_mod_gene\\":\\"\\",\\"search_gene\\":\\"\\",\\".clientdata_output_samplesize_den_width\\":440,\\".clientdata_output_samplesize_den_height\\":255,\\".clientdata_output_study_stat_width\\":595,\\".clientdata_output_study_stat_height\\":255,\\".clientdata_output_sample_stat_width\\":615,\\".clientdata_output_sample_stat_height\\":255,\\".clientdata_output_stepbystep_hidden\\":true,\\".clientdata_output_samplesize_den_hidden\\":false,\\".clientdata_output_study_stat_hidden\\":false,\\".clientdata_output_sample_stat_hidden\\":false,\\".clientdata_output_warn_hidden\\":false,\\".clientdata_output_warn2_hidden\\":false,\\".clientdata_output_warn3_hidden\\":false,\\".clientdata_output_warn4_hidden\\":false,\\".clientdata_output_warn5_hidden\\":false,\\".clientdata_output_warn_ont_hidden\\":false,\\".clientdata_output_warn_ont2_hidden\\":false,\\".clientdata_output_datatable_hidden\\":false,\\".clientdata_output_geo_acc_ui_hidden\\":true,\\".clientdata_output_metadata_column_hidden\\":true,\\".clientdata_output_full_metacol_hidden\\":true,\\".clientdata_output_downloadmeta_hidden\\":true,\\".clientdata_output_counts_sample_ui_hidden\\":true,\\".clientdata_output_downloadcounts_hidden\\":true,\\".clientdata_output_cor_sample_ui_hidden\\":true,\\".clientdata_output_show_grp_ui_hidden\\":true,\\".clientdata_output_corr_prop_hidden\\":true,\\".clientdata_output_draw_cor_ui_hidden\\":true,\\".clientdata_output_dens_sample_ui_hidden\\":true,\\".clientdata_output_draw_dens_ui_hidden\\":true,\\".clientdata_output_heat_prop_hidden\\":true,\\".clientdata_output_cluster_meth_hidden\\":true,\\".clientdata_output_cluster_genes_hidden\\":true,\\".clientdata_output_download_hidden\\":true,\\".clientdata_output_downloadscr_hidden\\":true,\\".clientdata_output_pca_prop_hidden\\":true,\\".clientdata_output_subprop2_ui_hidden\\":true,\\".clientdata_output_subprop_ui_hidden\\":true,\\".clientdata_output_legendplot_hidden\\":true,\\".clientdata_output_geo_summary_hidden\\":true,\\".clientdata_output_metadata_hidden\\":true,\\".clientdata_output_metadata_full_hidden\\":true,\\".clientdata_output_ontology_hidden\\":true,\\".clientdata_output_counts_warn_hidden\\":true,\\".clientdata_output_counts_text_hidden\\":true,\\".clientdata_output_counts_hidden\\":true,\\".clientdata_output_boxplot_counts_hidden\\":true,\\".clientdata_output_downloadqc_hidden\\":true,\\".clientdata_output_multiqc_hidden\\":true,\\".clientdata_output_corr_warn_hidden\\":true,\\".clientdata_output_corplot_ui_hidden\\":true,\\".clientdata_output_dens_warn_hidden\\":true,\\".clientdata_output_densityp_hidden\\":true,\\".clientdata_output_heat_warn_hidden\\":true,\\".clientdata_output_HeatStatic_hidden\\":true,\\".clientdata_output_HeatStaticscr_hidden\\":true,\\".clientdata_output_iheatmapui_hidden\\":true,\\".clientdata_output_downloadPCA2D_hidden\\":true,\\".clientdata_output_p2D_ui_hidden\\":true,\\".clientdata_output_pca_sam_warn_hidden\\":true,\\".clientdata_output_pca3Dplot_ui_hidden\\":true,\\".clientdata_output_t2Dplot_ui_hidden\\":true,\\".clientdata_output_tsne3Dplot_ui_hidden\\":true,\\".clientdata_output_geo_acc2_ui_hidden\\":true,\\".clientdata_output_metadata_analysis_hidden\\":true,\\".clientdata_output_group1_hidden\\":true,\\".clientdata_output_group2_hidden\\":true,\\".clientdata_output_ana_type_hidden\\":true,\\".clientdata_output_cov_hidden\\":true,\\".clientdata_output_meta_ui_hidden\\":true,\\".clientdata_output_gen_reg_sig_ui_hidden\\":true,\\".clientdata_output_heat_typeui_mod_regsig_hidden\\":true,\\".clientdata_output_heat_prop_mod_regsig_hidden\\":true,\\".clientdata_output_cluster_meth_mod_regsig_hidden\\":true,\\".clientdata_output_cluster_genes_mod_regsig_hidden\\":true,\\".clientdata_output_downloadfit_mod_regsig_hidden\\":true,\\".clientdata_output_downloadscr_mod_regsig_hidden\\":true,\\".clientdata_output_fdrui_mod_regsig_hidden\\":true,\\".clientdata_output_effect_ma_mod_ui_hidden\\":true,\\".clientdata_output_power_mod_ui_hidden\\":true,\\".clientdata_output_alpha_mod_ui_hidden\\":true,\\".clientdata_output_fitscreen_mod_regsig_hidden\\":true,\\".clientdata_output_scroll_mod_regsig_hidden\\":true,\\".clientdata_output_iheatmapui_mod_regsig_hidden\\":true,\\".clientdata_output_downloadMA_hidden\\":true,\\".clientdata_output_maplotui_mod_regsig_hidden\\":true,\\".clientdata_output_ma_mod_text_hidden\\":true,\\".clientdata_output_maplot_clickedpoints_hidden\\":true,\\".clientdata_output_gene_mod_warn_hidden\\":true,\\".clientdata_output_gene_sym_mod_warn_hidden\\":true,\\".clientdata_output_gene_sym_mod_green_hidden\\":true,\\".clientdata_output_det_mod_label_hidden\\":true,\\".clientdata_output_gen_det_mod_dl_hidden\\":true,\\".clientdata_output_plotdetui_mod_regsig_hidden\\":true,\\".clientdata_output_det_mod_text_hidden\\":true,\\".clientdata_output_det_mod_gene_clickedpoints_hidden\\":true,\\".clientdata_output_downloadsigdata_hidden\\":true,\\".clientdata_output_ui_reg_ilincs_option_hidden\\":true,\\".clientdata_output_power_sig_ui_hidden\\":true,\\".clientdata_output_alpha_sig_ui_hidden\\":true,\\".clientdata_output_dynamiclink_hidden\\":true,\\".clientdata_output_prop_power_ui_hidden\\":true,\\".clientdata_output_group1_pow_ui_hidden\\":true,\\".clientdata_output_group2_pow_ui_hidden\\":true,\\".clientdata_output_meta_pow_ui_hidden\\":true,\\".clientdata_output_gen_meta_pow_ui_hidden\\":true,\\".clientdata_output_samples1_hidden\\":true,\\".clientdata_output_samples2_hidden\\":true,\\".clientdata_output_effect_hidden\\":true,\\".clientdata_output_alpha_hidden\\":true,\\".clientdata_output_depth_ui_hidden\\":true,\\".clientdata_output_power_ui_hidden\\":true,\\".clientdata_output_det_effect_ui_hidden\\":true,\\".clientdata_output_fdr_ui_hidden\\":true,\\".clientdata_output_prop_detect_ui_hidden\\":true,\\".clientdata_output_group1_detect_ui_hidden\\":true,\\".clientdata_output_group2_detect_ui_hidden\\":true,\\".clientdata_output_gen_pow_btn_ui_hidden\\":true,\\".clientdata_output_gen_detect_btn_ui_hidden\\":true,\\".clientdata_output_sig1_warn_hidden\\":true,\\".clientdata_output_rank_warn_hidden\\":true,\\".clientdata_output_level_warn_hidden\\":true,\\".clientdata_output_sig_meta_hidden\\":true,\\".clientdata_output_sig_data_hidden\\":true,\\".clientdata_output_meta_pow_txt_hidden\\":true,\\".clientdata_output_meta_power_warn_hidden\\":true,\\".clientdata_output_metadata_pow_table_hidden\\":true,\\".clientdata_output_power1_warn_hidden\\":true,\\".clientdata_output_powerplot_hidden\\":true,\\".clientdata_output_gene_warn_hidden\\":true,\\".clientdata_output_gene_sym_warn_hidden\\":true,\\".clientdata_output_gene_sym_green_hidden\\":true,\\".clientdata_output_power2_warn_hidden\\":true,\\".clientdata_output_detect_gene_ui_hidden\\":true,\\".clientdata_output_gen_det_text_hidden\\":true,\\".clientdata_output_detect_gene_tab_hidden\\":true,\\".clientdata_output_steps_hidden\\":true,\\".clientdata_output_process_log_text_hidden\\":true,\\".clientdata_pixelratio\\":1,\\".clientdata_url_protocol\\":\\"http:\\",\\".clientdata_url_hostname\\":\\"www.ilincs.org\\",\\".clientdata_url_port\\":\\"\\",\\".clientdata_url_pathname\\":\\"/apps/grein/\\",\\".clientdata_url_search\\":\\"?gse=\\",\\".clientdata_url_hash_initial\\":\\"\\",\\".clientdata_url_hash\\":\\"\\",\\".clientdata_singletons\\":\\"991a9dee8b6f41039f0d48e0f3cb481dfef69248,abbc5191535f7e95ba1254171676097ac26b5edc,2581d36bc09732ee052ca2bcc5a7ae809019206f,2936dd0d01d731dd90358375bdaef9c44ce3b494,603e796bcfc2ab3685167d58c426f64c15a95192\\",\\".clientdata_allowDataUriScheme\\":true}}"]'

    def overview_streaming_updata(self):
        return '["2#0|m|{\\"method\\":\\"update\\",\\"data\\":{\\".clientdata_output_warn_hidden\\":true,\\".clientdata_output_warn3_hidden\\":true,\\".clientdata_output_warn4_hidden\\":true,\\".clientdata_output_warn5_hidden\\":true}}"]'

    def overview_form_data(self, no_datasets):
        return f"draw=1&columns%5B0%5D%5Bdata%5D=0&columns%5B0%5D%5Bname%5D=&columns%5B0%5D%5Bsearchable%5D=true&columns%5B0%5D%5Borderable%5D=true&columns%5B0%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B0%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B1%5D%5Bdata%5D=1&columns%5B1%5D%5Bname%5D=&columns%5B1%5D%5Bsearchable%5D=true&columns%5B1%5D%5Borderable%5D=true&columns%5B1%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B1%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B2%5D%5Bdata%5D=2&columns%5B2%5D%5Bname%5D=&columns%5B2%5D%5Bsearchable%5D=true&columns%5B2%5D%5Borderable%5D=true&columns%5B2%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B2%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B3%5D%5Bdata%5D=3&columns%5B3%5D%5Bname%5D=&columns%5B3%5D%5Bsearchable%5D=true&columns%5B3%5D%5Borderable%5D=true&columns%5B3%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B3%5D%5Bsearch%5D%5Bregex%5D=false&columns%5B4%5D%5Bdata%5D=4&columns%5B4%5D%5Bname%5D=&columns%5B4%5D%5Bsearchable%5D=true&columns%5B4%5D%5Borderable%5D=true&columns%5B4%5D%5Bsearch%5D%5Bvalue%5D=&columns%5B4%5D%5Bsearch%5D%5Bregex%5D=false&start=0&length={no_datasets}&search%5Bvalue%5D=&search%5Bregex%5D=false&search%5BcaseInsensitive%5D=true&search%5Bsmart%5D=true&escape=false"

    @staticmethod
    def get_random_url_string_parameter():
        return ''.join(random.choice(string.ascii_letters) for _ in range(10)) + "/" + \
               ''.join(random.choice(string.ascii_letters) for _ in range(3)) + "/" + \
               ''.join(random.choice(string.ascii_letters) for _ in range(8))

    @staticmethod
    def get_random_nonce_parameter():
        return ''.join(random.choice(string.ascii_letters) for _ in range(10))



import re
import requests
import json
import logging
from . import utils
from .exceptions import GreinLoaderException

LOGGER = logging.getLogger(__name__)
MAX_GREIN_DATASETS = 1000000

def load_overview(no_datasets = None) -> list:
    """ loads overview of the number of datasets given as parameter
        :param: no_samples: int, default parameter are all datasets on grein
        :type: no_datasets: int
        :return: list of dict, each dict is one dataset in GREIN
                 containing GEO id, number of samples, Species, title and summary
        :rtype: list_overview:list of dictionaries
    """
    if no_datasets == None:
        LOGGER.debug("Requesting all Datasets from GREIN")
        no_datasets = MAX_GREIN_DATASETS

    payloads = utils.GreinLoaderUtils()
    n = utils.GreinLoaderUtils.get_random_url_string_parameter()
    # create the unique random string used later for nonce parameter in url
    random_str = utils.GreinLoaderUtils.get_random_nonce_parameter()

    # base url
    grein_url = "http://www.ilincs.org/apps/grein/"

    # xhr_streaming_url will always be used for streaming requests in the code
    xhr_streaming_url = f"http://www.ilincs.org/apps/grein/__sockjs__/n={n}/xhr_streaming"

    # xhr_send_url will always be used for streaming requests in the code
    xhr_send_url = f"http://www.ilincs.org/apps/grein/__sockjs__/n={n}/xhr_send"


    LOGGER.debug("Requesting Session")
    s = requests.session()
    try:
        r = s.get(grein_url)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("GREIN not available with: ", grein_url)
        raise GreinLoaderException(f"Failed to contact GREIN at {grein_url}: ", err)
    LOGGER.debug("Connected to GREIN")

    # streaming request is necessary for connection parameters
    try:
        xhr_streaming_r = s.post(xhr_streaming_url, stream=True)
        lines = xhr_streaming_r.iter_lines()  # saves information provided by the streaming response
        xhr_streaming_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    for line in lines:  # decodes content of provided by the streaming request
        line_content = line.decode()
        if ("hhhhhhhhhh" in line_content):  # streaming content for the request is done and the connection initialized
            break
    try:
        xhr_send_r = s.post(xhr_send_url, data='["0#0|o|"]')
        xhr_send_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    config = None
    ACK_flag = False
    for line in lines:
        line_content = line.decode()
        if line_content.startswith('a["0#0|m|'):  # search for config parameter to parse sessionId
            config_string = line_content[9:len(
                line_content) - 2].replace('\\"', '"')
            config = json.loads(config_string)
        if "ACK" in line_content:
            ACK_flag = True
            break

    if not ACK_flag:
        LOGGER.error("Streaming Error")
        raise GreinLoaderException("Streaming Error no ACK")

    session_id = config["config"]["sessionId"]

    # requesting streaming parameter for overview page on GREIN
    try:
        xhr_send_r = s.post(xhr_send_url, data=payloads.overview_streaming())
        xhr_send_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error with: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    try:
        xhr_send_r = s.post(xhr_send_url, data=payloads.overview_streaming_updata())
        xhr_send_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error with: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    url_overview = f"http://www.ilincs.org/apps/grein/session/{session_id}/dataobj/datatable?w=&nonce={random_str}"
    # requesting overview of dataset with number of datasets defined in the data parameter, given in as method parameter

    try:
        overview_stream = s.post(url_overview, data=payloads.overview_form_data(no_datasets))
        overview_stream.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Overview streaming error", err)
        raise GreinLoaderException("Overview streaming error", err)

    # process dataset
    content_ = json.loads(overview_stream.content.decode())
    content_data = content_['data']  # containing relevant data in streaming response

    # creating datastructure for return, list of dictionaries, each dict is a dataset on GREIN
    overview = {}
    list_overview = []
    for item in content_data:  # iterates list of processed return content
        d = {
            "geo_accession": _format_geo_accession(item[0]),
            "no_samples": item[1],
            "species": item[2],
            "title": item[3],
            "study_summary": item[4]
        }
        list_overview.append(d)

    return list_overview


def _format_geo_accession(geo_accession_id):
    """ helper function for formating the Geo accession id
        :param: geo_accession_id: geo accession of GRIEN Dataset
        :return: formatted geo accession for a GREIN Dataset
    """
    s = re.search("GSE[0-9]{3,}", geo_accession_id)  # searches for geo accession with at least 3 digits
    if s is not None:
        return s.group()
    else:
        return ""


# the load_dataset(gse_id) loads the description, metadata and raw count matrix from GREIN based on the GSE id
# return values:
# description: dict
# metadata: dict
# count matrix: pandas dataframe

import io
import re
import requests
import random
import string
import logging
import json
import pandas
from typing import Tuple
from .exceptions import GreinLoaderException
from . import utils

LOGGER = logging.getLogger(__name__)


def load_dataset(gse_id: str) -> Tuple[dict, dict, pandas.DataFrame]:
    """ Loads a dataset from GREIN.
        :param: gse_id: The dataset's GSE id.
        :type: gse_id: str
        :return: description, metadata, count_matrix of the GREIN dataset
        :rtype: description:dict, metadata:dictionary, count_matrix:pandas dataframe
    """
    payloads = utils.GreinLoaderUtils(gse_id)
    # create the unique random string used later for nonce parameter in url
    n = utils.GreinLoaderUtils.get_random_url_string_parameter()

    # xhr_streaming_url will always be used for streaming requests in the code
    xhr_streaming_url = f"http://www.ilincs.org/apps/grein/__sockjs__/n={n}/xhr_streaming"

    # xhr_send_url will always be used for streaming requests in the code
    xhr_send_url = f"http://www.ilincs.org/apps/grein/__sockjs__/n={n}/xhr_send"

    # base url
    grein_url = "http://www.ilincs.org/apps/grein/"

    LOGGER.debug("Requesting Session")
    s = requests.session()  # requests a session on GREIN, cookies are provided within the session
    try:
        r = s.get(grein_url)
        r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("GREIN not available with: ", grein_url)
        raise GreinLoaderException(f"Failed to contact GREIN at {grein_url}: ", err)
    LOGGER.debug("Connected to GREIN")

    # streaming request is necessary for connection parameters
    try:
        xhr_streaming_r = s.post(xhr_streaming_url, stream=True)
        lines = xhr_streaming_r.iter_lines()  # saves information provided by the streaming response
        xhr_streaming_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    for line in lines:  # decodes content of provided by the streaming request
        line_content = line.decode()
        if ("hhhhhhhhhh" in line_content):  # streaming content for the request is done and the connection initialized
            LOGGER.debug("Connection initialized")
            break

    # streaming request for configs and sessionId
    try:
        xhr_send_r = s.post(xhr_send_url, data='["0#0|o|"]')
        xhr_send_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    config = None
    ACK_flag = False
    for line in lines:
        line_content = line.decode()
        if line_content.startswith('a["0#0|m|'):  # search for config parameter to parse sessionId
            config_string = line_content[9:len(
                line_content) - 2].replace('\\"', '"')
            config = json.loads(config_string)
        if "ACK" in line_content:
            ACK_flag = True
            break

    if not ACK_flag:
        LOGGER.error("Streaming Error")
        raise GreinLoaderException("Streaming Error no ACK")

    session_id = config["config"]["sessionId"]

    # streaming parameters of GREIN ui init for data set with gse_id
    try:
        xhr_send_summary_r = s.post(xhr_send_url,
                                    data=payloads.ui_init_parameter())  # data needs the gse_id in the payload str
        xhr_send_summary_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error with: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    for line in lines:
        line_content = line.decode()  # decode content from GREIN

    LOGGER.debug("Opening new connection")
    try:
        xhr_streaming_r = s.post(xhr_streaming_url, stream=True)
        lines = xhr_streaming_r.iter_lines()
        xhr_streaming_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error with: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    try:
        LOGGER.debug("Streaming parameter")
        xhr_send_r = s.post(xhr_send_url,
                            data=payloads.method_update_parameter())  # sets initial parameters for the dataset
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error with: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    for line in lines:
        line_content = line.decode()
        if "ACK" in line_content:
            break

    try:
        LOGGER.debug("Streaming client parameter")
        xhr_send_r = s.post(xhr_send_url, data=payloads.client_parameter())  # sets client parameter for dataset
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error with: ", err)
        raise GreinLoaderException("Streaming error: ", err)

    try:
        LOGGER.debug("Streaming dataset")
        s.post(xhr_send_url, payloads.stream_dataset_parameter())  # sets parameter for streaming
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error")
        raise GreinLoaderException("Streaming error: ", err)

    for line in lines:
        line_content = line.decode()
        if "ACK " in line_content:
            LOGGER.debug("Data received from GREIN ")
            break

    # random string must be created for the nonce parameter in the following requests
    random_str = utils.GreinLoaderUtils.get_random_nonce_parameter()
    # the description is requested for the dataset,     
    try:
        LOGGER.debug("Request Dataset")
        description_r = s.post(
            f"http://www.ilincs.org/apps/grein/session/{session_id}/dataobj/geo_summary?w=&nonce={random_str}",
            headers={
                "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
                "Accept": "application/json, text/javascript, */*; q=0.01",
                "Origin": "http://www.ilincs.org",
                "Referer": "http://www.ilincs.org/apps/grein/?gse=" + gse_id
            },
            data=payloads.description_formdata(100))
        description_r.raise_for_status()
    except requests.exceptions.HTTPError as err:
        LOGGER.error(f"Dataset description for {gse_id} not received: ", err)
        raise GreinLoaderException(f"Dataset description for {gse_id} not received: ", err)

    # request necessary for the metadata labels, provided in the ui via streaming
    try:
        metadata_labels_r = s.post(xhr_send_url, data=payloads.metadata_labels_parameter())
    except requests.exceptions.HTTPError as err:
        LOGGER.error(f"Metadata labels for {gse_id} not received: ", err)
        raise GreinLoaderException(f"Metadata labels for {gse_id} not received: ", err)

    ui_content = []
    for line in lines:  # the streaming request provides elements elements from the ui
        line_content = line.decode()
        ui_content.append(line_content)
        if "ACK" in line_content:
            break

    # parsing the provided data from streaming for keys in the metadata, later used for the metadata dictionary
    meta_data_labels = _parse_metadata(ui_content)
    data_samples = json.loads(description_r.content.decode())
    data_content = data_samples["data"]
    sample_content = data_content[1]
    no_of_samples = sample_content[1]
    metadata_formdata = _generate_metadata_formdata(len(meta_data_labels), no_of_samples)

    # random string created for requesting the metadata
    random_str = ''.join(random.choice(string.ascii_letters) for _ in range(10))
    # metadata request, without the keys for later
    try:
        metadata_r = s.post(
            f"http://www.ilincs.org/apps/grein/session/{session_id}/dataobj/metadata_full?w=&nonce={random_str}]",
            headers={
                "Content-Type": "application/x-www-form-urlencoded; charset=UTF-8",
                "Accept": "application/json, text/javascript, */*; q=0.01",
                "Origin": "http://www.ilincs.org",
                "Referer": "http://www.ilincs.org/apps/grein/?gse=" + gse_id
            }, data=metadata_formdata)
    except requests.exceptions.HTTPError as err:
        LOGGER.error(f"Metadata for {gse_id} not received: ", err)
        raise GreinLoaderException(f"Metadata for {gse_id} not received: ", err)

    # method update for count_matrix
    try:
        xhr_send_r = s.post(xhr_send_url, data=payloads.count_matrix_parameter())
    except requests.exceptions.HTTPError as err:
        LOGGER.error("Streaming error", err)
        raise GreinLoaderException("Streaming error: ", err)

    for line in lines:
        line_content = line.decode()
        if "ACK" in line_content:
            break

    # requesting count matrix
    try:
        count_matrix_r = s.post(f"http://www.ilincs.org/apps/grein/session/{session_id}/download/downloadcounts?w=")
    except requests.exceptions.HTTPError as err:
        LOGGER.error(f"Count Matrix for {gse_id} not received ", err)
        raise GreinLoaderException(f"Count Matrix for {gse_id} not received: ", err)

    LOGGER.debug("Count matrix received")

    # Before returning description, metadata and count_matrix is formatted,
    # before formatting all content from the requests must be decoded

    # formats the description with hidden method in the package,
    # the description is formatted in a dictionary containing the Study Link, Species, Title and Summary
    description = ""
    metadata = ""
    count_matrix = ""
    if description_r.status_code != 500:
        description = _format_description(
            json.loads(description_r.content.decode()))  # streaming content must be decoded

    # formats metadata to a dictionary with labels provided by metadata_labels_r and values provided by metadata_r
    if metadata_r.status_code != 500:   
        metadata = json.loads(metadata_r.content.decode())
        metadata = _format_metadata(metadata, meta_data_labels)

    # formats the count matrix provided by count_matrix_r request to a pandas dataframe
    if count_matrix_r.status_code != 500:
        count_matrix_string = count_matrix_r.content.decode()
        count_matrix_string = count_matrix_string.split("\n")
        count_matrix = pandas.read_csv(io.StringIO('\n'.join(count_matrix_string)), sep=",")
        # rename the first column name with "gene"
        count_matrix.rename(columns={count_matrix.columns[0]: str("gene")}, inplace=True)
    return description, metadata, count_matrix


def _format_description(description):
    """
    Formats raw description input from streaming request to a dictionary.
    :param: description:
    :type: description: str
    :return: description, keys: Study link, Species, Title, Summary
    :rtype: description:dict
    """
    d = {}
    data = description["data"]
    for i in data:
        if i[0] == 'Study link':
            d["Study link"] = re.search(
                "https?://(www\.)?[-a-zA-Z0-9@:%._+~#=]{1,256}\.[a-zA-Z0-9()]{1,6}([-a-zA-Z0-9()@:%_+.~#?&/=]*)",
                i[1]).group()
        if i[0] == 'Species':
            d["Species"] = "" if i[1] == 'character(0)' else i[1]
        if i[0] == 'Title':
            d["Title"] = "" if i[1] == 'character(0)' else i[1]
        if i[0] == 'Summary':
            d["Summary"] = "" if i[1] == 'character(0)' else i[1]
    return d


def _format_metadata(metadata, metadata_labels):
    """
    Formats raw metadata and metadata labels provided by streaming to create metadata dictionary
    :param: metadata, metadata_labels
    :type: metadata: str, metadata_labels: list
    :return: metadata
    :rtype: metadata:dict
    """
    mdict = {}
    data = metadata["data"]
    for item in data:
        item_dict = dict(zip(metadata_labels, item))
        mdict[item[1]] = item_dict
    return mdict


def _parse_metadata(stream_list):
    """
    parses raw metadata provided by streaming,
    :param: stream_list
    :type: stream_list: list
    :return: item_list
    :rtype: item_list: list
    """
    item_string = ""
    for i in stream_list:
        if i.find("<table class") != -1:
            item_string = i
            break
    n = item_string.replace("\\", "")
    item_list = re.findall("<th>(.*?)</th>", n)
    return item_list


def _generate_metadata_formdata(n_columns, no_samples=100):
    """
    generates formdata for metadata
    :param: number of columns used for metadata
    :type: n_columns: int
    :return: raw form data parameter for request
    :rtype: raw_form: string
    """
    raw_utils = utils.GreinLoaderUtils("")
    raw_form = raw_utils.raw_form_start()
    n = 1
    while n < n_columns-5:
        raw_form += raw_utils.raw_form_column(n)
        n = n+1
    raw_form += raw_utils.raw_form_end(no_samples)
    return raw_form




import unittest
import logging
import requests
import grein_loader as loader

LOGGER = logging.getLogger(__name__)


class TestDataset(unittest.TestCase):
    geo_accession = "GSE112749"
    geo_accession_2 = "GSE100075"

    def test_connection_to_grein(self):
        LOGGER.info("Test GREIN connection")
        r = requests.get("http://www.ilincs.org/apps/grein/")
        LOGGER.info("GREIN Connection with status code: ", r.status_code)
        self.assertEqual(r.status_code, 200, "Error GREIN not available with http://www.ilincs.org/apps/grein/")

    def test_dataset(self):
        LOGGER.info("Test dataset with geo accession: ", self.geo_accession)
        r = requests.get("http://www.ilincs.org/apps/grein/?gse=GSE112749")
        LOGGER.warning(f"Grein dataset {self.geo_accession} with status code: { r.status_code }")
        self.assertEqual(r.status_code, 200, f"Error GREIN dataset {self.geo_accession} not available")

    def test_dataset_request(self):
        LOGGER.info(f"Test GREIN dataset with GeoId: {self.geo_accession}")
        description, metadata, count_matrix = loader.load_dataset(self.geo_accession)
        self.assertIsNotNone(description)
        self.assertIsNotNone(metadata)
        self.assertIsNotNone(count_matrix)

    def test_dataset_request_2(self):
        LOGGER.info(f"Test GREIN dataset with GeoId: {self.geo_accession_2}")
        description, metadata, count_matrix = loader.load_dataset(self.geo_accession_2)
        self.assertIsNotNone(description)
        self.assertIsNotNone(metadata)
        self.assertIsNotNone(count_matrix)

    def test_metadata_label(self):
        LOGGER.info(f"Test GREIN metadata with GeoId: {self.geo_accession}")
        description, metadata, count_matrix = loader.load_dataset(self.geo_accession)
        self.assertIsNotNone(metadata)
        self.assertIsNotNone(description)
        self.assertIsNotNone(count_matrix)

    def test_overview(self):
        LOGGER.info("Test overview of GREIN datasets")
        overview = loader.load_overview(10)
        self.assertEqual(10, len(overview))
